' Include libraries
Imports System.IO

Public Class frmMain

    ' The file we're connected to
    Public FilePath As String, NickName As String

    ' Whether or not we'll exit the file watch loop
    Public EndLoop As Boolean, IgnoreChange As Boolean, HasChanged As Boolean

    ' Used to connect to a file
    Public Sub Connect(ByVal sFilePath As String, ByVal sNickName As String)

        ' Make sure the path at least exists
        If Not Directory.Exists(Path.GetDirectoryName(sFilePath)) Then
            Throw New DirectoryNotFoundException("Cannot find directory.")
        End If

        ' Firstly, initiate the file watcher
        fsWatch.Filter = Path.GetFileName(sFilePath)
        fsWatch.Path = Path.GetDirectoryName(sFilePath)

        ' Then, save the variables
        FilePath = sFilePath
        NickName = sNickName

        ' Set the caption
        Me.Text = Path.GetFileName(sFilePath) & " - " & NickName

        ' If the file dosen't exits, ...
        If Not File.Exists(FilePath) Then
            ' ... then we'll have to set the default text.
            txtLog.SaveFile(FilePath)
        End If

        ' Clear variables
        EndLoop = False

        ' Then, open the file
        LoadFile()

        ' Inform others that we have joined
        AppendFile("* " & NickName & " joined the server." & Environment.NewLine, True)

        ' Watch the file
        FileWatchLoop()

    End Sub

    Private Sub FileWatchLoop()

        ' Run the loop until we're expected to end
        Do Until EndLoop

            ' If we actually have an event, ...
            If HasChanged Then

                ' Load the file again
                LoadFile()

                ' Clear the variable
                HasChanged = False

            End If

            ' Wait for a millisecond
            System.Threading.Thread.Sleep(10)

            ' Do events
            Application.DoEvents()

        Loop

    End Sub

    Private Sub LoadFile()

        ' Read the text
        txtLog.LoadFile(FilePath)

    End Sub

    Private Sub AppendFile(ByVal Text As String, ByVal UseDefaultStyle As Boolean)

        ' Firstly, append the data to the log
        txtLog.AppendText(Text)

        ' Select the added text
        txtLog.Select(txtLog.TextLength - Text.Length + 1, Text.Length)

        ' Then set the size and font of that text
        If UseDefaultStyle Then
            txtLog.SelectionFont = Me.Font
            txtLog.SelectionColor = Color.Red
        Else
            txtLog.SelectionFont = txtAdd.Font
            txtLog.SelectionColor = txtAdd.ForeColor
        End If

        ' Then save the file
        txtLog.SaveFile(FilePath)

    End Sub

    Private Sub frmMain_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed

        ' Close the loop
        EndLoop = True
        IgnoreChange = True

        ' Inform that we have disconnected
        AppendFile("* " & NickName & " has left the server." & Environment.NewLine, True)

        ' Show the logon form
        frmLogon.Show()

    End Sub

    Private Sub frmMain_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize

        Dim iWidth As Int32, iHeight As Int32

        ' Retrieve the inner widths and heights
        iWidth = Me.ClientSize.Width
        iHeight = Me.ClientSize.Height

        ' Resize the controls
        txtLog.Width = iWidth - txtLog.Left - 8
        txtLog.Height = iHeight - txtLog.Top - txtAdd.Height - 16
        txtAdd.Top = txtLog.Top + txtLog.Height + 8
        txtAdd.Width = iWidth - txtAdd.Left - cmdSend.Width - 16
        cmdSend.Left = txtAdd.Left + txtAdd.Width + 8
        cmdSend.Top = txtAdd.Top
        cmdFont.Top = cmdSend.Top + cmdSend.Height + 1
        cmdFont.Left = cmdSend.Left

    End Sub

    Private Sub SendText()

        ' No point updating the file
        IgnoreChange = True

        ' And save this text to the file
        AppendFile(NickName & ": " & txtAdd.Text & Environment.NewLine, False)

        ' Return to watching the file
        IgnoreChange = False

        ' Clear the textbox
        txtAdd.Text = ""
        txtAdd.Focus()

    End Sub

    Private Sub cmdSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSend.Click

        ' Send the text
        SendText()

    End Sub

    Private Sub txtAdd_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles txtAdd.KeyDown

        ' See if the user pressed enter with no shift
        If e.KeyCode = Keys.Enter And Not e.Shift Then

            ' Send the text
            SendText()

            ' Stop this key from reaching the control
            e.SuppressKeyPress = True

        End If

    End Sub

    Private Sub cmdFont_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdFont.Click

        ' Show the font dialog
        If dialogFont.ShowDialog() = Windows.Forms.DialogResult.OK Then

            ' Set the fonts in both controls
            txtAdd.Font = dialogFont.Font
            txtAdd.ForeColor = dialogFont.Color

        End If

    End Sub

    Private Sub fsWatch_Changed(ByVal sender As Object, ByVal e As System.IO.FileSystemEventArgs) Handles fsWatch.Changed

        ' Inform that the document has been changed
        If Not IgnoreChange Then
            HasChanged = True
        End If

    End Sub

End Class
